structure Ast =
struct

  (* An expression *)
  datatype exp = 
    Num of int
  | Plus of exp * exp
  | Sub of exp * exp
  | Paren of exp


  fun toString e = 
    let
	fun mkString_two s e1 e2 = 
	    s ^ "(" ^ (toString e1) ^ "," ^ (toString e2) ^ ")"

	fun mkString_one s e = s ^ "(" ^ (toString e) ^ ")"
    in
	case e of
	    Num n =>  ("Num " ^ Int.toString n)
	  | Plus  (e1,e2) => mkString_two "Plus " e1 e2
	  | Sub   (e1,e2) => mkString_two "Sub " e1 e2
	  | Paren e1 => mkString_one "" e1
    end
 
  fun toPrettyString e = 
      let
      in
	  case e of
	      Num n =>  
	      Int.toString n
	    | Plus  (e1,e2) => 
	      "(" ^ (toPrettyString e1) ^ "+"  ^ (toPrettyString e2) ^ ")"
	    | Sub   (e1,e2) => 
	      "(" ^ (toPrettyString e1) ^ "-"  ^ (toPrettyString e2) ^ ")"
	    | Paren e1 => 
	      "(" ^ toPrettyString e1 ^ ")"
      end

  fun eval e = 
      case e of
	  Num n =>  n
	| Plus  (e1,e2) => (eval e1) + (eval e2)
	| Sub   (e1,e2) => (eval e1) - (eval e2)
	| Paren e1 => eval e1

end
